@@ -49,6 +49,9 @@ _create_txt_from_str
 _create_txt_from_list
     This function allows to generate a text file with synchronised signals from the input files.
 
+_pad_data:
+    Function to pad data with either padding type 'same' or 'zero'
+
 Observations/Comments
 ---------------------
 None
@@ -99,6 +102,7 @@ def synchronise_signals(in_signal_1, in_signal_2):
         The second signal synchronised.
     """
 
+    # signal normalisation
     mean_1, std_1, mean_2, std_2 = [np.mean(in_signal_1), np.std(in_signal_1), np.mean(in_signal_2),
                                     np.std(in_signal_2)]
     signal_1 = in_signal_1 - mean_1
@@ -106,38 +110,46 @@ def synchronise_signals(in_signal_1, in_signal_2):
     signal_2 = in_signal_2 - mean_2
     signal_2 /= std_2
 
+    # zero padding signals so that they are of same length, this facilitates the calculation because
+    # then the delay between both signals can be directly calculated
+    # zero padding only if needed
+    if (len(signal_1) != len(signal_2)):
+
+        # check which signal has to be zero padded
+        if (len(signal_1) < len(signal_2)):
+
+            # pad first signal
+            signal_1 = np.append(signal_1, np.zeros(len(signal_2) - len(signal_1)))
+
+        else:
+
+            # pad second signal
+            signal_2 = np.append(signal_2, np.zeros(len(signal_1) - len(signal_2)))
+
     # Calculate the full cross-correlation between the two signals.
     correlation = np.correlate(signal_1, signal_2, 'full')
 
-    # Finding the edge of the correct correlation signal
-    center = len(correlation) - len(signal_1) if len(signal_1) < len(signal_2) else len(correlation) - len(signal_2)
+    # crop signals to original length (removing zero padding)
+    signal_1 = signal_1[:len(in_signal_1)]
+    signal_2 = signal_2[:len(in_signal_2)]
 
-    # Finding the position of the maximum value of the correlation signal
-    max_position = correlation.argmax()
-    # Calculating the difference between the center and the position of the maximum value
-    phase_straight = center - max_position
+    # calculate tau / shift between both signals
+    tau = int(np.argmax(correlation) - (len(correlation)) / 2)
 
-    # Finding the position of the maximum value of the reversed signal (corr[::-1])
-    max_position_reversed = correlation[::-1].argmax()
-    # Calculating the difference between the center and the position of the maximum value in the reversed correlation
-    # signal
-    phase_reversed = center - max_position_reversed
+    # check which signal has to be sliced
+    if (tau < 0):
+        # tau negative --> second signal lags
+        signal_2 = signal_2[np.abs(tau):]
 
-    # Calculate the dephasing between the signals. Maximum value of both results guarantees that we find the true
-    # dephasing of the signals
-    phases_aux = [phase_straight, phase_reversed]
-    phase = np.abs(phases_aux).argmax()
-    true_phase = np.abs(phases_aux[phase])
-
-    if phases_aux[0] < phases_aux[1]:
-        signal_1 = signal_1[true_phase:]
-    else:
-        signal_2 = signal_2[true_phase:]
+    elif (tau > 0):
+        # tau positive ---> firs signal lags
+        signal_1 = signal_1[np.abs(tau):]
 
+    # revert signals to orignal scale
     result_signal_1 = signal_1 * std_1 + mean_1
     result_signal_2 = signal_2 * std_2 + mean_2
 
-    return true_phase, result_signal_1, result_signal_2
+    return tau, result_signal_1, result_signal_2
 
 
 def generate_sync_txt_file(in_path, channels=("CH1", "CH1"), new_path='sync_file.txt'):
@@ -322,7 +334,7 @@ def create_synchronised_files(in_path=(('file1.txt', 'file2.txt'), ('file1.h5',
     generate_sync_txt_file(in_path[0], channels=txt_channels, new_path=file_name + '.txt')
 
 
-def create_android_sync_header(in_path):
+def create_android_sync_header(in_path, sampling_rate):
 
     '''
     function in order to creating a new header for a synchronised android sensor file
@@ -330,6 +342,7 @@ def create_android_sync_header(in_path):
     Parameters
     ----------
     in_path (list of strings): list containing the paths to the files that are supposed to be synchronised
+    sampling_rate(int): The sampling rate to which the signals are going to be synchronised
 
     Returns
     -------
@@ -362,29 +375,34 @@ def create_android_sync_header(in_path):
                 # convert header into a dict
                 curr_header = json.loads(header_string)
 
+                # get the key 'internal sensor' or 'sensores internos'
+                dict_key = list(header.keys())[0]
+
                 # add the fields from the header of the current file
-                header['internal sensors']['sensor'].extend(curr_header['internal sensors']['sensor'])  # sensor field
-                header['internal sensors']['column'].extend(
-                    curr_header['internal sensors']['column'][1:])  # column field
+                header[dict_key]['sensor'].extend(curr_header[dict_key]['sensor'])  # sensor field
+                header[dict_key]['column'].extend(
+                    curr_header[dict_key]['column'][1:])  # column field
 
                 # get the last channel from the channel field
-                num_channels = header['internal sensors']['channels'][-1]
+                num_channels = header[dict_key]['channels'][-1]
 
                 # get the channels from the current sensor
-                new_channels = curr_header['internal sensors']['channels']
+                new_channels = curr_header[dict_key]['channels']
 
                 # adjust the channel number
                 new_channels = [ch + (num_channels + 1) for ch in new_channels]
 
-                header['internal sensors']['channels'].extend(new_channels)  # channels field
-                header['internal sensors']['label'].extend(
-                    curr_header['internal sensors']['label'])  # label field
-                header['internal sensors']['resolution'].extend(
-                    curr_header['internal sensors']['resolution'])  # resolution field
-                header['internal sensors']['special'].extend(
-                    curr_header['internal sensors']['special'])  # special field
-                header['internal sensors']['sleeve color'].extend(
-                    curr_header['internal sensors']['sleeve color'])  # sleeve color field
+                header[dict_key]['channels'].extend(new_channels)  # channels field
+                header[dict_key]['label'].extend(
+                    curr_header[dict_key]['label'])  # label field
+                header[dict_key]['resolution'].extend(
+                    curr_header[dict_key]['resolution'])  # resolution field
+                header[dict_key]['special'].extend(
+                    curr_header[dict_key]['special'])  # special field
+                header[dict_key]['sleeve color'].extend(
+                    curr_header[dict_key]['sleeve color'])  # sleeve color field
+                header[dict_key]['sampling rate'] = sampling_rate # changing sampling rate to the one specified by the
+                                                                  # user
 
     # create new header string
     header_string = "# OpenSignals Text File Format\n# " + json.dumps(header) + '\n# EndOfHeader\n'
@@ -569,6 +587,10 @@ def pad_android_data(sensor_data, report, start_with=None, end_with=None, paddin
                     sig_channel = np.pad(sig_channel, (start_pad.size, end_pad.size), 'constant',
                                          constant_values=(0, 0))
 
+                else:  # undefined input
+
+                    raise IOError('Invalid padding type. Please choose between padding types \'same\' or \'zero\'.')
+
             # concatenate the channel to the padded data
             padded_data = np.vstack((padded_data, sig_channel))
 
@@ -579,7 +601,7 @@ def pad_android_data(sensor_data, report, start_with=None, end_with=None, paddin
     return padded_sensor_data
 
 
-def save_synchronised_android_data(time_axis, data, header, path, file_name='android_synchroinsed'):
+def save_synchronised_android_data(time_axis, data, header, path, file_name='android_synchronised'):
     """
     Function used for saving synchronised android data into a single file
 
@@ -587,14 +609,15 @@ def save_synchronised_android_data(time_axis, data, header, path, file_name='and
     ----------
     time_axis (N,  array_like): The time axis after the padding and re-sampling the sensor data.
 
-    data (list): List containing the padded and re-sampled sensor signals. The length of data along the 0-axis has to be
-                 the same size as time_axis
+    data(list of arrays or list of list): List containing the padded and re-sampled sensor signal arrays or lists.
+                                          The length of each signal array/list along the 0-axis has to be the same size
+                                          as time_axis.
 
    header (string): A string containing the header that is supposed to be added to the file.
 
     path (string): A string with the location where the file should be saved.
 
-    file_name (string, optional): The name of the file, with the suffix '.txt'. If not specified, the file is named
+    file_name (string, optional): The name of the file. If not specified, the file is named
                              'android_synchronised.txt'.
 
     """
@@ -657,9 +680,6 @@ def sync_android_files(in_path, out_path, sync_file_name='android_synchroinzed',
 
     """
 
-    # create new file header
-    new_header = create_android_sync_header(in_path)
-
     # load the data
     sensor_data, report = load_android_data(in_path, print_report=True)
 
@@ -716,7 +736,7 @@ def sync_android_files(in_path, out_path, sync_file_name='android_synchroinzed',
         # inform the user
         print('Synchronizing from start of {} sensor until end of {} sensor.'.format(report['starting order'][-1],
                                                                                      report['stopping order'][0]))
-        print('Using padding type: same.')
+        print('Using padding type: \'same\'.')
 
         padded_sensor_data = pad_android_data(sensor_data, report)
 
@@ -830,9 +850,9 @@ def sync_android_files(in_path, out_path, sync_file_name='android_synchroinzed',
         sampling_rate = _round_sampling_rate(report['max. sampling rate'])
 
         # inform the user
-        print('The signals will be re-sampled to a sampling rate of {} Hz.'.format(sampling_rate))
+        print('The signals will be re-sampled to:  {} Hz.'.format(sampling_rate))
         print('Shifting the time axis to start at zero and converting to seconds.')
-        print('Using interpolation type: previous.')
+        print('Using interpolation type: \'previous\'.')
 
         # cycle over the sig
         for data in padded_sensor_data:
@@ -847,6 +867,9 @@ def sync_android_files(in_path, out_path, sync_file_name='android_synchroinzed',
     # ---- Saving data to file ---- #
     print('\n---- Saving Data to file ----\n')
 
+    # create new file header
+    new_header = create_android_sync_header(in_path, sampling_rate)
+
     # save the data to the file
     save_path = save_synchronised_android_data(re_sampled_time[0], re_sampled_data, new_header, out_path,
                                                file_name=sync_file_name)
@@ -862,7 +885,8 @@ def sync_android_files(in_path, out_path, sync_file_name='android_synchroinzed',
 
 def _shape_array(array1, array2):
     """
-    Function that equalises the input arrays by zero-padding the shortest one.
+    Function that equalises the input arrays by padding the shortest one using padding type 'same', i.e. replicating
+    the last row.
 
     ----------
     Parameters
@@ -877,22 +901,30 @@ def _shape_array(array1, array2):
     arrays: numpy.array
         Array containing the equal-length arrays.
     """
-    if len(array1) > len(array2):
-        new_array = array2
-        old_array = array1
-    else:
-        new_array = array1
-        old_array = array2
 
-    length = len(old_array) - len(new_array)
+    # check if the data arrays are of different size
+    if (len(array1) != len(array2)):
 
-    for i in range(length):
-        n = new_array[-1].copy()
-        n[0::3] += 1
-        n[2::3] = 0
-        new_array = np.vstack([new_array, [n]])
+        # check which array needs to be padded
+        if (len(array1) < len(array2)):
+
+            # get the length of the padding
+            pad_length = len(array2) - len(array1)
+
+            # pad the first array
+            array1 = _pad_data(array1, pad_length)
+
+        else:
+
+            # get the length of the padding
+            pad_length = len(array1) - len(array2)
+
+            # pad the second array
+            array2 = _pad_data(array2, pad_length)
+
+    # hstack both arrays / concatenate both array horizontally
+    arrays = np.hstack([array1, array2])
 
-    arrays = np.hstack([old_array, new_array])
     return arrays
 
 
@@ -1011,6 +1043,7 @@ def _create_txt_from_str(in_path, channels, new_path):
 
     data = []
     nr_channels = []
+    is_integer_data = True
     for file in files:
         for i, device in enumerate(file.keys()):
             nr_channels.append(len(list(file[device])))
@@ -1018,17 +1051,29 @@ def _create_txt_from_str(in_path, channels, new_path):
 
     dephase, s1, s2 = synchronise_signals(data[0], data[1])
 
-    new_header = [h.replace("\n", "") for h in header]
-    sync_file = open(new_path, 'w')
-    sync_file.write(' \n'.join(new_header) + '\n')
+    if data[0] is not int or data[1] is not int:
+        is_integer_data = False
 
+    # Avoid change in float precision if we are working with float numbers.
+    if not is_integer_data:
+        round_data_0 = [float('%.2f' % (value)) for value in data[0]]
+        round_data_1 = [float('%.2f' % (value)) for value in data[1]]
+        round_s_1 = [float('%.2f' % (value)) for value in s1]
+        round_s_2 = [float('%.2f' % (value)) for value in s2]
+    else:
+        round_data_0 = data[0]
+        round_data_1 = data[1]
+        round_s_1 = s1
+        round_s_2 = s2
+
+    # Check which array is aligned.
     old_columns = np.loadtxt(in_path)
-    if np.array_equal(s1, data[0]):
+    if np.array_equal(round_s_1, round_data_0):
         # Change the second device
         aux = 3 * nr_channels[0]
         columns = old_columns[dephase:, aux:]
         new_file = _shape_array(old_columns[:, :aux], columns)
-    elif np.array_equal(s2, data[1]):
+    elif np.array_equal(round_s_2, round_data_1):
         # Change the first device
         aux = 3 * nr_channels[1]
         columns = old_columns[dephase:, :aux]
@@ -1036,8 +1081,18 @@ def _create_txt_from_str(in_path, channels, new_path):
     else:
         print("The devices are synchronised.")
         return
+
+    # write header to file
+    new_header = [h.replace("\n", "") for h in header]
+    sync_file = open(new_path, 'w')
+    sync_file.write(' \n'.join(new_header) + '\n')
+
+    # write data to file
     for line in new_file:
-        sync_file.write('\t'.join(str(int(i)) for i in line) + '\t\n')
+        if is_integer_data:
+            sync_file.write('\t'.join(str(int(i)) for i in line) + '\t\n')
+        else:
+            sync_file.write('\t'.join(str(i) for i in line) + '\t\n')
     sync_file.close()
 
 
@@ -1055,40 +1110,121 @@ def _create_txt_from_list(in_path, channels, new_path):
     new_path : str
         Path to create the new file.
     """
+
     header = ["# OpenSignals Text File Format"]
     files = [bsnb.load(p) for p in in_path]
     with open(in_path[0], encoding="latin-1") as opened_p:
         with open(in_path[1], encoding="latin-1") as opened_p_1:
+            # append both headers
             header.append(opened_p.readlines()[1][:-2] + ', ' + opened_p_1.readlines()[1][3:])
+
     header.append("# EndOfHeader")
 
+    # lists for holding the read data
     data = []
     nr_channels = []
+
+    # read the data
     for i, file in enumerate(files):
         nr_channels.append(len(list(file)))
         data.append(file[channels[i]])
 
-    dephase, s1, s2 = synchronise_signals(data[0], data[1])
+    # calculate the delay between both signals
+    dephase, _, _ = synchronise_signals(data[0], data[1])
 
+    # load original data
+    data_1 = np.loadtxt(in_path[0])
+    data_2 = np.loadtxt(in_path[1])
+
+    # Check which device lags
+    if dephase < 0:
+
+        # second device lags
+        # slice the data
+        data_2 = data_2[np.abs(dephase):]
+
+    elif dephase > 0:
+
+        # first device lags
+        # slice the data
+        data_1 = data_1[np.abs(dephase):]
+
+    else:
+        # dephase == 0 ---> devices were already syncronised
+        print("The devices were already synchronised.")
+
+    # pad data so that both devices are of the same length
+    # in case that phase = 0 the data will only be concatenated horizontally
+    new_file = _shape_array(data_1, data_2)
+
+    # write header to file
     new_header = [h.replace("\n", "") for h in header]
     sync_file = open(new_path, 'w')
     sync_file.write('\n'.join(new_header) + '\n')
 
-    if np.array_equal(s1, data[0]):
-        # Change the second device
-        old_columns = np.loadtxt(in_path[1])
-        columns = old_columns[dephase:]
-        new_file = _shape_array(columns, np.loadtxt(in_path[0]))
-    elif np.array_equal(s2, data[1]):
-        # Change the first device
-        old_columns = np.loadtxt(in_path[0])
-        columns = old_columns[dephase:]
-        new_file = _shape_array(columns, np.loadtxt(in_path[1]))
-    else:
-        print("The devices are synchronised.")
-        return
+    # writing synchronised data to file
     for line in new_file:
-        sync_file.write('\t'.join(str(int(i)) for i in line) + '\t\n')
+        sync_file.write('\t'.join(str(i) for i in line) + '\t\n')
+
+    # close the file
     sync_file.close()
 
 
+def _pad_data(data, pad_length, padding_type='same'):
+    """
+    Function for padding data. The function uses padding type 'same', i.e. it replicates the last row of the data, as default
+
+    ----------
+    Parameters
+    ----------
+    data (numpy.array): the data that is supposed to be padded
+
+    pad_length (int): the length of the padding that is supposed to be applied to data
+
+    padding_type (string, optional): The type of padding applied, either: 'same' or 'zero'. Default: 'same'
+
+    Return
+    ------
+    padded_data (numpy.array): The data with the padding
+    """
+
+    # get the sampling period (or distance between sampling points, for PLUX devices this is always 1)
+    # it is assumed that the signals are equidistantly sampled therefore only the distance between to sampling points
+    # is needed to calculate the sampling period
+    T = data[:, 0][1] - data[:, 0][0]
+
+    if padding_type == 'same':
+
+        # create the 'same' padding array
+        padding = np.tile(data[-1, 1:], (pad_length, 1))
+
+    elif padding_type == 'zero':
+
+        # get the number of columns for the zero padding
+        num_cols = data.shape[1] - 1  # ignoring the time/sample column
+
+        # create the zero padding array
+        padding = np.zeros((pad_length, num_cols))
+
+    else:
+
+        IOError('The padding type you chose is not defined. Use either \'same\ or \'zero\'.')
+
+    # create the time / sample axis that needs to be padded
+    start = data[:, 0][-1] + T
+    stop = start + (T * pad_length)
+    time_pad = np.arange(start, stop, T)
+    time_pad = time_pad[:pad_length]  # crop the array if there are to many values
+
+    # expand dimension for hstack operation
+    time_pad = np.expand_dims(time_pad, axis=1)
+
+    # hstack the time_pad and the zero_pad to get the final padding array
+    pad_array = np.hstack((time_pad, padding))
+
+    # vstack the pad_array and the new_array
+    padded_data = np.vstack([data, pad_array])
+
+    return padded_data
+
+
